import os

script_dir = os.path.dirname(__file__)  # Script directory
aba_lookup_table_path = os.path.join(script_dir, 'data/aba_lookup_table.csv')

open(aba_lookup_table_path, 'w')

with open('temp/unchecked_aba_lookup_table.csv', 'r') as in_file, open(aba_lookup_table_path, 'w') as out_file:
    seen = set() # set for fast O(1) amortized lookup
    for line in in_file:
        if line in seen: continue # skip duplicate

        seen.add(line)
        out_file.write(line)


import re
import numpy as np
import pandas as pd

filename = "temp/unfiltered_websites_list.txt"
data = np.loadtxt(filename, delimiter='\n', dtype=str)
filtered = []
for i in range(len(data)):
    value = re.findall("(https://usbanksdirectory.com/routing-numbers/([^0-9]*)/)", data[i])
    if value:
        filtered.append(value[0][0])

filtered = pd.DataFrame(filtered)
filtered.to_csv("websites_list.csv")



# Generates list of ABA numbers from bank-code.net website
# https://python.plainenglish.io/send-http-requests-as-fast-as-possible-in-python-304134d46604
# https://usbanksdirectory.com/routing-numbers/

# Import libraries
import csv
import pandas as pd
import time
import re
import asyncio
import aiohttp
from aiohttp.client import ClientSession

open('temp/unchecked_aba_lookup_table.csv', 'w')

headerList = ['aba_number', 'address', 'state', 'phone_number', 'bank']   # adding header

# open CSV file and assign header
with open("temp/unchecked_aba_lookup_table.csv", 'w') as file:
    dw = csv.DictWriter(file, delimiter=',',
                        fieldnames=headerList)
    dw.writeheader()

async def parse_website(url:str, start:float, session:ClientSession):
    async with session.get(url) as response:

        time_since_start = time.time() - start

        page = await response.text()

        table = pd.read_html(page, index_col=0)  # aba table parsed from
        table_df = pd.DataFrame(table[0])

        bank_name = table_df.iat[0,0]
        bank_name = re.search('(([A-Z][A-Z]*)\s)*', bank_name)
        bank_name = bank_name.group(0)
        table_df = table_df[2:]
        table_df.insert(3, '', bank_name)


        table_df.to_csv('temp/unchecked_aba_lookup_table.csv', mode='a', index=True, header=False)

        print(f"T+{'%.2f'%time_since_start} {url}")


async def parse_all_websites(urls:str, start:float):
    my_conn = aiohttp.TCPConnector(limit=10)
    async with aiohttp.ClientSession(connector=my_conn) as session:
        tasks = []
        for url in urls:
            task = asyncio.ensure_future(parse_website(url=url, start=start, session=session))
            tasks.append(task)
        await asyncio.gather(*tasks,return_exceptions=True)   # the await must be nest inside of the session

websites_list = pd.read_csv (r'temp/websites_list.csv')
websites_list = websites_list.iloc[:,1]
# final_list = pandas.DataFrame()

start = time.time()
asyncio.run(parse_all_websites(websites_list, start))
end = time.time()
print(f'Logged {len(websites_list)} website tables in {end - start} seconds')


import pandas as pd
import os

script_dir = os.path.dirname(__file__)  # Script directory
aba_lookup_table_path = os.path.join(script_dir, 'data/aba_lookup_table.csv')


# Parses an ACH number and returns information about it
def parse_aba(bank_number):
    bank_number = str(bank_number)
    aba_lookup_table = pd.read_csv(aba_lookup_table_path)

    if is_aba(bank_number):
        for i in range(len(aba_lookup_table)):
            if aba_lookup_table.aba_number[i] == bank_number:
                return (list(aba_lookup_table.iloc[i]))

# Parses a user-defined number to see if it is an ABA number
# https://en.wikipedia.org/wiki/ABA_routing_transit_number
def is_aba(bank_number):
    output = False
    bank_number = str(bank_number)
    digits = [int(i) for i in bank_number]
    length = len(bank_number)

    if length == 9:
        if((3 * (digits[0] + digits[3] + digits[6]) +
             7 * (digits[1] + digits[4] + digits[7]) +
             (digits[2] + digits[5] + digits[8]))) % 10 == 0:
            output = True
    else:
        output = False
    return output


# Parses a user-defined number to see if it is an ACH number
def is_eti(bank_number):
    output = False
    bank_number = str(bank_number)
    digits = [int(i) for i in bank_number]
    length = len(bank_number)
    first_two_digits = digits[0] + digits[1]
    first_two_digits = int(first_two_digits)

    if is_aba(bank_number):
        if first_two_digits >= 61 | first_two_digits <= 72:
            output = True
    else:
        output = False

    return output
